//------------------------------------------------------------------
/*
    
Controls.c
    
Routines to perform automatic control of APD bias voltage and the time expansion
    
*/
//------------------------------------------------------------------
// Constants and global variables
#include <constants.h>
#include <externals.h>
// C libraries
#include <propeller.h>
#include <stdio.h>
#include "propeller-icc.h"

//------------------------------------------------------------------
// Protoypes of external routines
// None
//------------------------------------------------------------------ START
void APD_Control(void)
/*
    Routine to control the noise on the APD by adjusting the bias voltage.
    A flag is set or cleared by the 'N' key to indicate control ON or OFF.
    The noise value is controlled between two setpoints read from the EEPROM..
    The gap between the two setpoints provides hysterysis.    
    The bias voltage is only changed when the results index is 1. This gives stability to the control loop.
*/

  {                                                                                                                                        
  if((int_settings[DATA_INDEX] == 1) && (int_settings[CONTROL_NOISE_FLAG] != 0))        // Check that the control loop is active         
    {                                                                                                                                            
                                                                                                                                                    
    if(spi_read[NOISE] > int_settings[CONTROL_NOISE_HI])                                    // If the measured APD noise is too high then
      int_settings[APD_VOLTAGE]--;                                                                                // lower the APD bias voltage
    else if(spi_read[NOISE] < int_settings[CONTROL_NOISE_LOW])                        // If the APD noise is too low then
      int_settings[APD_VOLTAGE]++;                                                                                // increase the APD bias voltage
            
    if (int_settings[APD_VOLTAGE] <50)                                                                        // Check for the lower control limit
      int_settings[APD_VOLTAGE] = 50;
    else if(int_settings[APD_VOLTAGE] > 250)                                                            // Check for the upper control limit
      int_settings[APD_VOLTAGE] = 250;
    }
  }
//------------------------------------------------------------------
void Tex_Control(void)
/*
    Routine to control the amount of time expansion by adjusting the frequency of the second crystal.
    A resolution setpoint is read from the EEPROM. If it is zero then the control loop is off.
    The resolution setpoint is specified in centimeters.
    If the setpoint is positive then forward time expansion results.
    If the setpoint is negative then reverse time expansion results.
    
    The actual control is done using the Span value because it has higher precision than the resolution.
    Control takes place between [setpoint] and [setpoint + 10%].
    The gap between the two setpoints provides hysterysis.
    
    The crystal frequency is only changed when the results index is 1. This gives stability to the loop.
*/

  {
    int con_span;
    
    if((int_settings[DATA_INDEX] == 1)& (int_settings[CONTROL_RESOLUTION_FLAG] == 1))    // Check that the control loop is active
        {                                                                                                                                                
        
                                                                                                                                                        // Test for a request to change the direction of the time expansion
        if((int_settings[CONTROL_RESOLUTION] < 0) & (direction == FORWARD))            // If current time expansion is forward but the request is for reverse then
            int_settings[SECOND_CRYSTAL] = 64;                                                                        // Jump to reverse time expansion
        if((int_settings[CONTROL_RESOLUTION] > 0) & (direction == REVERSE))            // If current time expansion is reverse but the request is for forward then
            int_settings[SECOND_CRYSTAL] = 192;                                                                        // Jump to forward time expansion
            
                                                                                                                                                        // All controlling is done using the span value
        con_span = (int)(range * 100 / int_settings[CONTROL_RESOLUTION]);                // Calculate the control span using the current range and control resolution
        if(con_span < 0)
            con_span = 0 - con_span;                                                                                            // Include the sign to acount for forward and reverse time expansion

    if(spi_read[SPAN] > (con_span + con_span / 10))                                                    // If the actual span > control span + 10% (both signed integers) then
            {
            if (int_settings[CONTROL_RESOLUTION] > 0)                                                            // If the requested time expansion is forward
          int_settings[SECOND_CRYSTAL]++;                                                                            // Increase the crystal frequency
            else
                int_settings[SECOND_CRYSTAL]--;                                                                            // For requested time expansion is reverse, decrease the crystal frequency
            }
            
    else if(spi_read[SPAN] < con_span)                                                                            // If the actual span < control span (both signals signed integers) then
            {
      if (int_settings[CONTROL_RESOLUTION] > 0)                                                            // If the requested time expansion is forward
          int_settings[SECOND_CRYSTAL]--;                                                                            // Decrease the crystal frequency
            else
                int_settings[SECOND_CRYSTAL]++;                                                                            // For requested time expansion is reverse, increase the crystal frequency
            }
    if (int_settings[SECOND_CRYSTAL] <10)                                                                        // Keep the crystal DAC voltage between 10 and 245 counts
      int_settings[SECOND_CRYSTAL] = 10;                                                                        // This limits the control range to between +9cm and -9cm
    if(int_settings[SECOND_CRYSTAL] > 245)
      int_settings[SECOND_CRYSTAL] = 245;
        }    
    }    
/*
-----------------------------------------------------------------------------------------------------------
This work is licensed under the Creative Commons Attribution 3.0 Unported License.
To view a copy of this license, visit http://creativecommons.org/licenses/by/3.0/
or send a letter to Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
-----------------------------------------------------------------------------------------------------------
*/
//------------------------------------------------------------------ END