//------------------------------------------------------------------
/*

Serial.c

Routines dealing with communications through the USB port.
See description of commands and display options in the: DS00 - Laser Range Finder Manual

*/
//------------------------------------------------------------------
// Constants and global variables
#include <constants.h>
#include <externals.h>
// C libraries
#include <propeller.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <math.h>
#include "propeller-icc.h"

//------------------------------------------------------------------
// Protoypes of external routines
void EEPROM_Editor(void);                        // EEPROM.c
void Initialise_Parameters(void);        // Utilities.c
float Rescm(void);                                    // Utilities.c
void Save_Memory(void);                            // Utilities.c

//------------------------------------------------------------------ START
char Echo_Key(void)
/*
    A function to get a character from the USB port, echo it and set it to lower case.
*/

    {
    int inkey;    
    inkey = getchar();                // Fetch character
    putchar(inkey);                        // Echo
    return(tolower(inkey));        // Drop to lower case and return
    }
//------------------------------------------------------------------        
void Print_Line(int length)
/*
    A display formatting function that prints a line using the '-' character.
*/

    {
    while(length > 0)                // Repeat until line is printed
        {
        putchar('-');                    // -
        length--;                            // Keep going
        }
    putchar('\r');                    // Put a <CR> at the end of the line
    }
//------------------------------------------------------------------
void Print_Float(float val)
/*
    Routine to print floating point numbers with 2 decimal places.
    The field width is fixed at 7 characters including the decimal point.
    Trialing zeros are displayed.
    This fixes a bug in the printf function.
*/
    {
    int a, b, sign;
    int i[6];
    
    i[3] = '.';
    if(val<0.0)
        sign = '-';
    else
        sign = ' ';
    
    a = val*100.0;                // Work to two decimal places and round to nearest integer

    if(a>99999)
        a = 99999;
    
    b = a/10000;                                            // Find 100's
    i[0] = b + '0';                                    // Make character
    a = a - b * 10000;
    
    b = a/1000;                                                // Find 10's
    i[1] = b + '0';                                    // Make character
    a = a - b * 1000;
    
    b = a/100;                                                // Find 1's
    i[2] = b + '0';                                    // Make character
    a = a - b * 100;
    
    b = a/10;                                                    // Find 0.1's
    i[4] = b + '0';                                    // Make character
    a = a - b * 10;
    
    i[5] = a + '0';
    
    b = 0;
    for(a=0; a<6; a++)
        {
        if((i[a] == '0') & (b == 0) & (a < 2))
            i[a] = ' ';
        else if(((i[a] > '0') | (a > 1)) & (b == 0))
            {
            b = 1;
            putchar(sign);
            }            
        putchar(i[a]);
        }
    }

//------------------------------------------------------------------        
void Settings_Display(void)
/*
    Displays the current settings.
*/

  {
    printf("\r      Settings Editor\r");                                // Heading    
    Print_Line(26);    
    printf("[G] Set Amplifier Gain %d\r[V] Set APD Voltage    %d\r", int_settings[AMPLIFIER_GAIN], int_settings[APD_VOLTAGE]);                                                                            // Separator    
    if(int_settings[CONTROL_NOISE_FLAG] == 0)
        printf("[N] Control APD Noise  OFF\r");
    else
        printf("[N] Control APD Noise  ON\r");
    Print_Line(26);    
    printf("[M] Set Maximum Range  %d\r[X] Set Timebase       %d\r", int_settings[MEASURING_RANGE], int_settings[SECOND_CRYSTAL]);
    if(int_settings[CONTROL_RESOLUTION_FLAG] == 0)
        printf("[R] Control Resolution OFF\r");
    else
        printf("[R] Control Resolution ON\r");
    Print_Line(26);                                                                            // Separator
    if(int_settings[AVERAGE_FLAG] == 0)
        printf("[A] Averaging          OFF\r");
    else
        printf("[A] Averaging          ON\r");
    
    printf("[T] Test Port          %d\r", int_settings[TEST_PORT]);
    }
//------------------------------------------------------------------
void Unprocessed_Display(void)
/*
    Displays the unprocessed data that comes from the DS00VQ100 chip.
*/

  {
    int j;
    int mask = 0x80;
    
    if(direction == FORWARD)                // Modify the Span display to indicate the time expansion direction
        j = spi_read[SPAN];                        // Forward time expansion results in a positive Span value
    else
        j = 0 - spi_read[SPAN];                // Reverse time expansion results in a negative Span value
            
    printf("ZF:%5d    ZR:%5d    | SF:%5d    SR:%5d    | SP:%5d    | AN:%5d    | ST:",
    spi_read[ZERO_FRONT], spi_read[ZERO_REAR], spi_read[SIGNAL_FRONT], spi_read[SIGNAL_REAR], j, spi_read[NOISE]);
    
                                                                    // Unpack the Status register to print as a binary number
    for(j=0; j<8; j++)                            // ANSI C does not have a binary print function
        {                                                         
        if((spi_read[STATUS] & mask) != 0)
            putchar('1');
        else
            putchar('0');
        mask = mask >> 1;
        }
        
    putchar('\r');                                     
  }
//------------------------------------------------------------------ 
void Distances_Display(void)
/*
    Shows the data from the DS00VQ100 chip converted into distance units.
*/

  {                                                                                        // Check the Status register to see if signal is valid
    if(valid_signal == TRUE)                                        // Signal is valid - wide enough for an accurate measurement
        {
        printf("DS:"); Print_Float(distance);
        printf("    | ZF:"); Print_Float(zero_front); 
        printf("    | SF:"); Print_Float(signal_front);
        printf("    SW:"); Print_Float(signal_width_out);
        }
    else                                                                                // Signal is invalid - either too small or lost altogether
        printf("DS: ---.--    | ZF: ---.--    | SF: ---.--    SW: ---.--");
    
    printf("    | MR:"); Print_Float(Rescm());
    putchar('\r');    
  }   
//------------------------------------------------------------------
void Fast_Display(void)
/*
    This display shows the distance in meters.
*/

  {
    if(valid_signal == TRUE)                                // Check the Status register to see if signal is valid
        {
        Print_Float(distance);
        printf(" m\r");                                                // Signal is valid - wide enough for an accurate measurement                    
        }
    else
        printf(" ---.-- m\r");                              // Signal is invalid - either too small or lost altogether
  }
//------------------------------------------------------------------ 
void Control_Display(void)
/*
    Shows information relating to the APD noise and time expansion control loops.
*/

  {
    if(int_settings[CONTROL_NOISE_FLAG] == 0)
        printf("NL:  0  NH:  0");
    else
        printf("NL:%3d  NH:%3d", int_settings[CONTROL_NOISE_LOW], int_settings[CONTROL_NOISE_HI]);
        
    printf("  AV:%3d  AN:%3d", int_settings[APD_VOLTAGE], spi_read[NOISE]);
    
    if(int_settings[CONTROL_RESOLUTION_FLAG] == 0)
        printf("  | CR:  0");
    else
        printf("  | CR:%3d", int_settings[CONTROL_RESOLUTION]);
        
    printf("  TX:%3d  MR:", int_settings[SECOND_CRYSTAL]); Print_Float(Rescm());    
    printf("  | AG:%3d  SW:", int_settings[AMPLIFIER_GAIN]); Print_Float(signal_width_in);    
    putchar('\r');
  }
//------------------------------------------------------------------
void Edit_Setting(int location)
/*
    A short text editor used to change the integer values of the settings display.
*/

    {
    char invalue[8];
    
    int inkey = 0;
    int charpointer = 0;

    memset(invalue,0,8);                                                // Clear value string for later use

    inkey = Echo_Key();                                                    // Fetch the first key
    while((inkey != 13) & (charpointer < 8))        // Keep getting keys until either <CR> or too many characters
        {                                                                                                                            
        invalue[charpointer] = inkey;                            // Store the key press
        inkey = Echo_Key();                                                // Fetch and echo the next key press
        charpointer++;                                                        // Point to the next storage location
        }

    if(charpointer > 0)                                                    // If a value is entered then continue
        int_settings[location] = atoi(invalue);        // Convert the value string into a number
                                                                                            // and store in the requested location
    }
//------------------------------------------------------------------  
void User_Action()
/*
    Certain key presses cause actions to take place immediately. These are not case sensitive.
*/

  {
  char inkey;
    
  inkey = global_key;                                                  // Fetch key press from another cog 
    inkey = tolower(inkey);                                            // Switch to lower case
    putchar(inkey);                                                            // Echo the key on the USB port
    switch(inkey)                                                                // Decode key press
        {
        case ' ':                                                                    // The space character pauses the readings
            inkey = getchar();                                            // Fetch character to wait for next key press which restarts the measurements
        break;
        case 'a':                                                                    // Switch averaging ON or OFF
            if(int_settings[AVERAGE_FLAG] == 0)
                {
                int_settings[AVERAGE_FLAG] = 1;
                printf(" = Averaging ON\r");
                start_flag = RESTART;
                }
            else
                {
                int_settings[AVERAGE_FLAG] = 0;
                printf(" = Averaging OFF\r");
                }
        break;
        case 'c':
            int_settings[DISPLAY_MODE] = 'c';                // Switch to control display
        break;
        case 'd':
            int_settings[DISPLAY_MODE] = 'd';                // Switch to distances display
        break;
        case 'e':
            EEPROM_Editor();                                                // Take the user to the EEPROM editor
        break;
        case 'f':
            int_settings[DISPLAY_MODE] = 'f';                // Switch to fast display mode where only the distance is shown
        break;
        case 'g':                                                                    // Set Amplifier gain
            printf(" = Amplifier Gain [%d]?", int_settings[AMPLIFIER_GAIN]);
            Edit_Setting(AMPLIFIER_GAIN);
        break;
        case 'i':                                                                    // Immediately load any new settings
            int_settings[FOOTER] = ASYNC_FOOTER;        // by changing the SPI footer
        break;
        case 'l':                                                                    // Laser toggle control
            if(int_settings[LASER_VOLTAGE] == 0)
                {
                int_settings[LASER_VOLTAGE] = 20;            // Turn on the laser
                start_flag = RESTART;
                printf(" = Laser ON\r");
                }
            else
                {
                int_settings[LASER_VOLTAGE] = 0;            // Turn off the laser
                printf(" = Laser OFF\r");
                }
            int_settings[FOOTER] = ASYNC_FOOTER;        // and immeditaley execute by changing the SPI footer
    break;
        case 'm':                                                                    // Select the measuring range
            printf(" = Maximum Range [%d]?", int_settings[MEASURING_RANGE]);
            Edit_Setting(MEASURING_RANGE);
            start_flag = RESTART;
        break;
        case 'n':                                                                    // Turn the APD noise control loop ON or OFF
            if(int_settings[CONTROL_NOISE_FLAG] == 0)
                {
                int_settings[CONTROL_NOISE_FLAG] = 1;
                printf(" = Control APD Noise ON\r");
                }
            else
                {
                int_settings[CONTROL_NOISE_FLAG] = 0;
                printf(" = Control APD Noise OFF\r");
                }
        break;
        case 'p':                                                                // Save the current settings to Permanent EEPROM memory
            Save_Memory();
        break;
        case 'r':                                                                // Turn the resolution control loop ON or OFF
            if((int_settings[CONTROL_RESOLUTION_FLAG] == 0) & (int_settings[CONTROL_RESOLUTION] != 0))
                {
                int_settings[CONTROL_RESOLUTION_FLAG] = 1;
                printf(" = Control Resolution ON\r");
                }
            else
                {
                int_settings[CONTROL_RESOLUTION_FLAG] = 0;
                printf(" = Control Resolution OFF\r");
                }
        break;        
        case 's':                                                                    // Switch to the settings display
            int_settings[DISPLAY_MODE] = 's';                
        break;
        case 't':                                                                    // Select the test port
            printf(" = Test Port [%d]?", int_settings[TEST_PORT]);
            Edit_Setting(TEST_PORT);
        break;
        case 'u':                                                                    // Switch to unprocessed data display
            int_settings[DISPLAY_MODE] = 'u';                
        break;
        case 'v':                                                                    // Change the APD bias voltage
            printf(" = APD Bias Voltage [%d]?", int_settings[APD_VOLTAGE]);
            Edit_Setting(APD_VOLTAGE);
        break;
        case 'x':                                                                    // Change the timebase expansion
            printf(" = Timebase [%d]?", int_settings[SECOND_CRYSTAL]);
            Edit_Setting(SECOND_CRYSTAL);
            start_flag = RESTART;
        break;
                
        default:
            putchar('?');                                                        // Invalid action
        break;
        }
    }
/*
-----------------------------------------------------------------------------------------------------------
This work is licensed under the Creative Commons Attribution 3.0 Unported License.
To view a copy of this license, visit http://creativecommons.org/licenses/by/3.0/
or send a letter to Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
-----------------------------------------------------------------------------------------------------------
*/
//------------------------------------------------------------------ END
