#ifndef _PROPELLER_ICC_H
#define _PROPELLER_ICC_H

#include <propeller.h>
#include <unistd.h>

// propmacros
//

/**
 * Chip Version address.
 */
#ifdef P8X32A
#define PROP_CLKFREQ_ADDR       0
#define PROP_CLKMODE_ADDR       4
#define PROP_CHIP_VERSION_ADDR  0xffff
#endif

/**
 * FALSE is a convenience macro representing zero symbolically.
 */
#ifndef FALSE
#define FALSE 0
#endif

/**
 * TRUE is a convenience macro representing non-zero symbolically.
 */
#ifndef TRUE
#define TRUE -1
#endif

/**
 * NULL is a convenience macro for c-string termination.
 */
#ifndef NULL
#define NULL (char*) 0
#endif

/**
 * Maximum negative number
 */
#ifndef MAXNEG
#define MAXNEG  (0x80000000)
#endif

/**
 * Maximum positive number
 */
#ifndef MAXPOS
#define MAXPOS  (0x7fffffff)
#endif

/**
 * Value for PI
 */
#ifndef PI
#define PI  3.141593
#endif

/**
 * Get absolute value of a number.
 * @param val is constant or variable
 */
#define ABS(val)    ((val < 0) ? -val : val)

/**
 * GetChip Version
 * This depends on the masked rom version.
 */
#define CHIPVER    (*(volatile unsigned char*)(PROP_CHIP_VERSION_ADDR))

/**
 * Get the integer value of a single bit 2^bit or 1<<bit.
 * @param bitnum contains bit number.
 * @returns integer value of bit power of 2.
 */
#define DECODE(bitnum) (1<<(bitnum))

/**
 * Get the most significant bit number represented by an integer value.
 * @see also ENCODE macro which will be faster, but input value not preserved.
 * @param value contains integer value to encode.
 * @returns most significant bit number power of 2 represented by value.
 */
int encode(int value);

/**
 * Get the most significant bit number represented by an integer value
 * @note This macro has restrictions on use; it can not be used as a parameter;
 * parameters passed to it must be variables.
 * @param val contains integer value to encode; val will not be preserved.
 * @param bitnum is most significant bit number power of 2 of value.
 * @returns bit number in the bitnum parameter.
 */
#define ENCODE(val,bitnum)     \
do {    \
    (bitnum) = 0;       \
    while((val) != 0) { \
        (bitnum)++;     \
        (val) >>= 1;    \
    }                   \
    (bitnum)--;         \
} while(0)


/**
 * Clear lock - use GETCARRY/GETZERO to get lock state.
 * @param lock is lock identifier and must be a variable;
 */
#define LOCKCLR(lock)     \
do {    \
    lockclr(lock);   \
} while(0)

/**
 * New lock - use GETCARRY/GETZERO to get lock state.
 * @param lock is lock identifier return and must be a variable;
 */
#define LOCKNEW(lock)     \
do {    \
    locknew(lock);   \
} while(0)

/**
 * Return lock to lock pool
 * @param lock is lock identifier and must be a variable;
 */
#define LOCKRET(lock)     \
do {    \
    lockret(lock);   \
} while(0)

/**
 * Set lock - use GETCARRY/GETZERO to get previous lock state.
 * @param lock is lock identifier and must be a variable
 */
#define LOCKSET(lock)     \
do {    \
    lockset(lock);   \
} while(0)

/**
 * Clear a bit in val
 * @param val variable where bit will be cleared
 * @param bit is bit number to clear.
 */
#define CLR(val,bit)     do { (val) &= ~(1 << (bit)); } while (0)

/**
 * Set a bit in val
 * @param val variable where bit will be set
 * @param bit is bit number to set.
 */
#define SET(val,bit)     do { (val) |= (1 << (bit)); } while (0)

/**
 * Flip a bit in val
 * @param val variable where bit will be toggled
 * @param bit is bit number to toggle.
 */
#define FLIP(val,bit)    do { (val) ^= (1 << (bit)); } while (0)


/**
 * pause program execution for number of milli-seconds
 * @param ms is number of milli-seconds to wait.
 */
#define msleep(ms)      do { usleep(1000*ms); } while (0)


#endif
